#
#                      compute_displacement.com
#
#
#--------------------------------------------------------------------------
# This script first runs the fftfault.f source code for calculation of surface
# displacement due to a number of arbitrarily oriented fault segments of varying
# locking depths.  This script also uses GMT3.4 commands for grid editing and 
# d isplay
#---------------------------------------------------------------------------
#
#
# fftfault code requires the following command line information:
#
#       fftfault D1 D2 Zobs ele.dat istr xd.grd yd.grd zd.grd
#
#  where 
#	 D1 		= depth to bottom of fault (negative, km)
#	 D2 		= depth to top of fault (negative, km)
#	 Zobs 		= depth of observation plane (negative and > D2, km)
#	 ele.dat 	= file containing fault element locations (x,y) and
#			  force components (Fx, Fy, Fz)
#	 istr 		= displacement/stress option - (0) displacement (U,V,W)
#						       (1) stress (Txx,Tyy,Txy)
#	 x,y,z.grd 	= output files (mm/yr)


# The following code first reads in all fault segments and evaluates
# displacement for a common model where all faults slip freely from a depth
# of -30 km to infinite depth. Then the  individual locking depth if each segment
# is superimposed by evaluating the model where the fault slips freely from -30 km 
# to its appropriate locking depth following results of  Table 1 (Smith and 
# Sandwell, in review 2003).  The sum of all fault segments is computed to form 
# the final grids.


fftfault -100000 -30 0 all_segs.dat 0 xall.grd yall.grd zall.grd

fftfault -30 -5.9 0 seg1.dat 0 x1.grd y1.grd z1.grd

	grdmath xall.grd x1.grd + = xout.grd
	grdmath yall.grd y1.grd + = yout.grd
	grdmath zall.grd z1.grd + = zout.grd

	\rm xall.grd yall.grd zall.grd x1.grd y1.grd z1.grd


fftfault -30 -6.3 0 seg2.dat 0 x2.grd y2.grd z2.grd

	grdmath xout.grd x2.grd + = xout.grd
	grdmath yout.grd y2.grd + = yout.grd
	grdmath zout.grd z2.grd + = zout.grd

	\rm x2.grd y2.grd z2.grd


fftfault -30 -22.6 0 seg3.dat 0 x3.grd y3.grd z3.grd
	
	grdmath xout.grd x3.grd + = xout.grd
	grdmath yout.grd y3.grd + = yout.grd
	grdmath zout.grd z3.grd + = zout.grd

	\rm x3.grd y3.grd z3.grd

fftfault -30 -2.0 0 seg4.dat 0 x4.grd y4.grd z4.grd
	
	grdmath xout.grd x4.grd + = xout.grd
	grdmath yout.grd y4.grd + = yout.grd
	grdmath zout.grd z4.grd + = zout.grd

	\rm x4.grd y4.grd z4.grd


fftfault -30 -13.1 0 seg5.dat 0 x5.grd y5.grd z5.grd
	
	grdmath xout.grd x5.grd + = xout.grd
	grdmath yout.grd y5.grd + = yout.grd
	grdmath zout.grd z5.grd + = zout.grd

	\rm x5.grd y5.grd z5.grd


fftfault -30 -26.0 0 seg6.dat 0 x6.grd y6.grd z6.grd
	
	grdmath xout.grd x6.grd + = xout.grd
	grdmath yout.grd y6.grd + = yout.grd
	grdmath zout.grd z6.grd + = zout.grd

	\rm x6.grd y6.grd z6.grd


fftfault -30 -25.2 0 seg7.dat 0 x7.grd y7.grd z7.grd
	
	grdmath xout.grd x7.grd + = xout.grd
	grdmath yout.grd y7.grd + = yout.grd
	grdmath zout.grd z7.grd + = zout.grd

	\rm x7.grd y7.grd z7.grd


fftfault -30 -12.7 0 seg8.dat 0 x8.grd y8.grd z8.grd
	
	grdmath xout.grd x8.grd + = xout.grd
	grdmath yout.grd y8.grd + = yout.grd
	grdmath zout.grd z8.grd + = zout.grd

	\rm x8.grd y8.grd z8.grd


fftfault -30 -14.5 0 seg9.dat 0 x9.grd y9.grd z9.grd
	
	grdmath xout.grd x9.grd + = xout.grd
	grdmath yout.grd y9.grd + = yout.grd
	grdmath zout.grd z9.grd + = zout.grd

	\rm x9.grd y9.grd z9.grd


fftfault -30 -1.3 0 seg10.dat 0 x10.grd y10.grd z10.grd
	
	grdmath xout.grd x10.grd + = xout.grd
	grdmath yout.grd y10.grd + = yout.grd
	grdmath zout.grd z10.grd + = zout.grd

	\rm x10.grd y10.grd z10.grd


fftfault -30 -9.3 0 seg11.dat 0 x11.grd y11.grd z11.grd
	
	grdmath xout.grd x11.grd + = xout.grd
	grdmath yout.grd y11.grd + = yout.grd
	grdmath zout.grd z11.grd + = zout.grd

	\rm x11.grd y11.grd z11.grd


fftfault -30 -19.4 0 seg12.dat 0 x12.grd y12.grd z12.grd
	
	grdmath xout.grd x12.grd + = xout.grd
	grdmath yout.grd y12.grd + = yout.grd
	grdmath zout.grd z12.grd + = zout.grd

	\rm x12.grd y12.grd z12.grd


fftfault -30 -1.6 0 seg13.dat 0 x13.grd y13.grd z13.grd
	
	grdmath xout.grd x13.grd + = xout.grd
	grdmath yout.grd y13.grd + = yout.grd
	grdmath zout.grd z13.grd + = zout.grd

	\rm x13.grd y13.grd z13.grd


fftfault -30 -13.7 0 seg14.dat 0 x14.grd y14.grd z14.grd
	
	grdmath xout.grd x14.grd + = xout.grd
	grdmath yout.grd y14.grd + = yout.grd
	grdmath zout.grd z14.grd + = zout.grd

	\rm x14.grd y14.grd z14.grd


fftfault -30 -9.1 0 seg15.dat 0 x15.grd y15.grd z15.grd
	
	grdmath xout.grd x15.grd + = xout.grd
	grdmath yout.grd y15.grd + = yout.grd
	grdmath zout.grd z15.grd + = zout.grd

	\rm x15.grd y15.grd z15.grd


fftfault -30 -15.7 0 seg16.dat 0 x16.grd y16.grd z16.grd
	
	grdmath xout.grd x16.grd + = xout.grd
	grdmath yout.grd y16.grd + = yout.grd
	grdmath zout.grd z16.grd + = zout.grd

	\rm x16.grd y16.grd z16.grd


fftfault -30 -18.9 0 seg17.dat 0 x17.grd y17.grd z17.grd
	
	grdmath xout.grd x17.grd + = xout.grd
	grdmath yout.grd y17.grd + = yout.grd
	grdmath zout.grd z17.grd + = zout.grd

	\rm x17.grd y17.grd z17.grd


fftfault -30 -12.3 0 seg18.dat 0 x18.grd y18.grd z18.grd
	
	grdmath xout.grd x18.grd + = xout.grd
	grdmath yout.grd y18.grd + = yout.grd
	grdmath zout.grd z18.grd + = zout.grd

	\rm x18.grd y18.grd z18.grd


# Edits size of grid display and gives appropriate POD lat,long axis

cut_grids.com xout.grd yout.grd zout.grd

# Plots grids in GMT3.4

cd plots
ln -s ../*final.grd .
make_plots.com -39 -34 -8 9 .75 .6 x_final
make_plots.com -39 -34 -8 9 .75 .6 y_final
make_plots.com -39 -34 -8 9 .75 .6 z_final

