c
      subroutine fterm(kx,ky,d1,d2,z,ux,uy,uz,
     +                               vy,vz,wz,
     +                               c3x,c3y,c3z)
c
c  input
c    kx  - x wavenumber        (1/km)
c    ky  - y wavenumber        (1/km)
c    d1  - bottom of fault in source or image (km)
c    d2  - top of fault in source or image (km)
c    z   - observation plane   (km)
c
c  output
c    cux,cuy,cuz  - elements of force/displacement matrix
c    cvy,cvz,cwz               (km**2/Pa)
c    c3x,c3y,c3z  - non-zero vertical stress at surface z=0
c                              (km)
c
      implicit complex (c)
      implicit real*4 (k)
c
c  routine to compute the greens function for a fault element
c
      common/elastic/rlam,rmu,rc,rd,alph,pi
c
      if (kx.eq.0.and.ky.eq.0.) return
c
      kh2=kx*kx+ky*ky
      kh=sqrt(kh2)
      B=2*pi*kh
      B2=B*B
c
      Bs1=B*abs(z-d1)
      Bs2=B*abs(z-d2)
      if(Bs1.gt.50.) then 
        es1=0.
      else
        es1=exp(-Bs1)
      endif
      if(Bs2.gt.50.) then 
        es2=0.
      else
        es2=exp(-Bs2)
      endif
c
      kxx=(kx*kx)/kh2
      kyy=(ky*ky)/kh2
      kxy=(kx*ky)/kh2
c
      ux=(rc/B2)*(es2*(rd+kyy-kxx*(1+Bs2))-es1*(rd+kyy-kxx*(1+Bs1)))
      uy=-(rc/B2)*kxy*(es2*(2.+Bs2)-es1*(2.+Bs1))
      uz=-(rc/B2)*(kx/kh)*(es2*(1.+Bs2)-es1*(1.+Bs1))
      vy= (rc/B2)*(es2*(rd+kxx-kyy*(1+Bs2))-es1*(rd+kxx-kyy*(1+Bs1)))
      vz=-(rc/B2)*(ky/kh)*(es2*(1.+Bs2)-es1*(1.+Bs1))
      wz= (rc/B2)*(es2*(rd+1.+Bs2)-es1*(rd+1.+Bs1))
c
c   compute the surface traction components
c
      rl2m=rlam+2.*rmu
      Bd1=-B*abs(d1)
      Bd2=-B*abs(d2)
      if(Bd1.lt.-50) then
        e1=0.
      else
        e1=exp(Bd1)
      endif
      if(Bd2.lt.-50) then
        e2=0.
      else
        e2=exp(Bd2)
      endif
      arg1=e1*(alph*Bd1-rlam/rl2m)
      arg2=e2*(alph*Bd2-rlam/rl2m)
      arg3=e1*(alph*Bd1-rmu/rl2m-2*alph)
      arg4=e2*(alph*Bd2-rmu/rl2m-2*alph)
      c3x=cmplx(0.,-(arg2-arg1)*kx/(B*kh))
      c3y=cmplx(0.,-(arg2-arg1)*ky/(B*kh))
      c3z=cmplx((arg4-arg3)/B,0.)
      return
      end
c
